\\ Copyright 2014, 2015 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

allocatemem(300 * 10^6);
default(strictargs,1);
read("pol-pfrac.gp");
read("recurrence-guess.gp");
want_mirror = 0;

\\ Strichartz and Wang
\\ polroots(p)
\\ vecsort(apply(abs,polroots(p)))
\\ vecsort(polroots(p),(a,b)->a=abs(a);b=abs(b);cmp(b,a))
\\ root 1.954776399102463907266
\\
sw_poly = x^9 - 3*x^8 + 3*x^7 - 3*x^6 + 2*x^5 + 4*x^4 - 8*x^3 + 8*x^2 - 16*x + 8;


\\-----------------------------------------------------------------------------
install(time, "lf");
elapsed_time__previous = time();
elapsed_time() =
{
  my(t = time(),
     delta = t - elapsed_time__previous);
  elapsed_time__previous = t;
  delta;
}

\\-----------------------------------------------------------------------------

read("hashtable2.gp");
poly_to_name_hash = ht_create();
poly_to_name(p) = if(ht_search(poly_to_name_hash,p),ht_value(poly_to_name_hash,p),"");
poly_to_name_store(p,name) = \
  poly_to_name_hash=ht_insert(poly_to_name_hash, p, name);

poly_to_name_store(sw_poly,"sw_poly");
poly_to_name_store(x^8 - x^7 + x^6 - 2*x^5 + x^4 - x^3 + x^2 - 2,"Eight");
poly_to_name_store(x^8     - 2*x^6         - x^4     + 2*x^2 - 4,"Eight-alt");
poly_to_name_store(x^11 - 2*x^10 + 3*x^9 - 5*x^8 + 6*x^7 - 9*x^6 + 10*x^5 - 14*x^4 + 12*x^3 - 14*x^2 + 8*x - 8,"Eleven");
poly_to_name_store(x^14 - 2*x^10 - 5*x^6 - 6*x^4 - 4*x^2 - 8,"Fourteen");
poly_to_name_store(x^18 - 5*x^17 + 12*x^16 - 21*x^15 + 28*x^14 - 25*x^13 + 8*x^12 + 19*x^11 - 53*x^10 + 90*x^9 - 112*x^8 + 118*x^7 - 108*x^6 + 88*x^5 - 48*x^4 + 16*x^3 + 8*x^2 - 32*x + 32,"Eighteen");

\\ poly_to_name_hash
\\ ht_search(poly_to_name_hash,p)
\\print("lookup: ",poly_to_name(sw_poly));

\\-----------------------------------------------------------------------------

cmp_by_abs(a,b) = cmp(abs(a),abs(b));
polroots_sorted(p) = vecsort(polroots(p),cmp_by_abs);

gf_terms(g,n) =
{
  my(x = variable(g),
     zeros = min(n,valuation(g,x)),
     v = Vec(g + O(x^n)));
  if(zeros>=0, concat(vector(zeros,i,0), v),
               v[-zeros+1 .. #v]);
}

\\-----------------------------------------------------------------------------

conf_to_string(conf) = concat(apply(b->Str(b),conf));

conf_print(conf) =
{
  print(if(conf[17],"--->","    ")," .",
        if(conf[9],"--->","    ")," .",
        if(conf[18],"--->","    "));
  print("    ",
        if(conf[7],"|"," "),
        if(conf[8],"^"," "),
        "    ",
        if(conf[10],"|"," "),
        if(conf[11],"^"," "));
  for(i=1,1, 
     print("    ",
           if(conf[7],"|"," "),
           if(conf[8],"|"," "),
           "    ",
           if(conf[10],"|"," "),
           if(conf[11],"|"," ")));
  print("    ",
        if(conf[7],"v"," "),
        if(conf[8],"|"," "),
        "    ",
        if(conf[10],"v"," "),
        if(conf[11],"|"," "));

  print(if(conf[6],"<---","    ")," .",
        if(conf[1],"<---","    "),". ",
        if(conf[12],"<---","    "),
        "  ",conf_to_string(conf),
        ", sides=",conf_num_sides(conf),
        if(conf_is_double_seg(conf),", double",""));
  print(if(conf[5],"--->","    "),"  ",
        if(conf[2],"--->","    "),"  ",
        if(conf[13],"--->","    "));

  print("    ",
        if(conf[4],"|"," "),
        if(conf[3],"^"," "),
        "    ",
        if(conf[15],"|"," "),
        if(conf[14],"^"," "));
  for(i=1,1, 
     print("    ",
           if(conf[4],"|"," "),
           if(conf[3],"|"," "),
           "    ",
           if(conf[15],"|"," "),
           if(conf[14],"|"," ")));
  print("    ",
        if(conf[4],"v"," "),
        if(conf[3],"|","."),
        "    ",
        if(conf[15],"v"," "),
        if(conf[14],"|","."));
  print("      ",if(conf[16],"<---",""));
  print("-----------------------------------------------");
}

\\ conf_num_sides(conf) = \
\\   if(conf[1], 1 + conf[8] + conf[9] + conf[10],  0);
conf_num_sides(conf) = \
  conf[1] + conf[8] + conf[9] + conf[10];
conf_is_single_seg(conf) = conf[1] && ! conf[2];
conf_is_double_seg(conf) = conf[1] && conf[2];
conf_is_empty(conf) = vecsum(conf)==0;
conf_is_full(conf) = (vector(#conf,i, if(i<17,conf[i],1)) == conf_all_ones);

conf_all_ones = vector(18,n, 1);
conf_print(conf_all_ones)

\\-----------------------------------------------------------------------------

\\ Numbering of Duvall and Keesling, plus 16,17,18
\\ 16 new triangle, then need 17,18
\\
\\ *---------*---------*---------*    *---------*---------*---------*
\\  \ 17|17 /|\  9|9  /|\ 18|18 /      \   |   /|\   |   /|\       /
\\   \     / | \  |  / | \     /        \  | 7/ | \8 |10/ | \11   /
\\    \   /  |  \ | /  |  \   /       17 \ | /17| 9\ | /9 |18\   / 18
\\     \ /  7|8  \ / 10|11 \ /            \ /   |   \ /   |   \ /
\\      *--- | ---*--  |  --*              *--- | ---*--  |  --*
\\     / \  7|8  / \ 10|11 / \            / \   |   / ^   |   / \
\\    /   \  |  /   \  |  /   \         6/   \6 | 1/   \1 |12/   \12
\\   /  |  \ | /  |  \ | /  |  \        /  | 7\ | /8 |10\ | /11|  \
\\  /  6|6  \|/  1|1  \|/ 12|12 \      /   |   \|v   |   \|/   |   \
\\ *---------*<--------*---------*    *---------*---------*---------*
\\  \  5|5  /|\  2|2  /|\ 13|13 /      \   |   /|\   |   /|\   |   /
\\   \  |  / | \  |  / | \  |  /        \  | 4/ | \3 |15/ | \14|  /
\\    \   /  |  \   /  |  \   /         5\   /5 | 2\   /2 |13\   /13
\\     \ /  4|3  \ / 15|14 \ /            \ /   |   \ /   |   \ /
\\      *--  |  --*--  |  --*              *--  |  --*--  |  --*
\\       \  4|3  / \ 15|14 /                \   |   / \   |   /
\\        \  |  /   \  |  /                  \  |16/   \16|  /
\\         \ | /  |  \ | /                   4\ | /3 |15\ | /14
\\          \|/ 16|16 \|/                      \|/   |   \|/
\\           *---------*                        *---------*
\\
\\ 6 -> left 2bit 2,9
\\ 

\\ expanded    1  2  3  4  5   6  7 8    9 19 11  12,13,14,15, 16,17,18
left_perm  = [ 8, 1, 9, 8,10,  9, 1,10, 15, 3, 2,  5, 4, 7, 6, 17,12,16 ];
right_perm = [10, 1,12,11,14, 13, 2,15,  3, 8, 1,  9, 8,10, 9, 18,16,6 ];

perms_vector = [left_perm,right_perm];
conf_permute(conf,perm) = vector(#conf,i, conf[perm[i]]);

\\              1  2   3  4  5   6  7  8  9 19 11  12,13,14,15, 16,17,18
mirror_perm = [ 1, 2, 15,14,13, 12,11,10, 9, 8, 7,  6, 5, 4, 3, 16,18,17 ];
identity_perm = vector(#mirror_perm,i,i);
conf_permute(mirror_perm,mirror_perm) == identity_perm || quit(1)

\\---------

\\ c=vector(15);
\\ c[6+1] = 1;
\\ c[9+1] = 1;
\\ c = conf_permute(c,left_perm);
\\ c
\\ conf_print(c)
\\ quit

conf_initial_vector = vector(#identity_perm,c, vector(#identity_perm,i, i==c));
\\ conf_initial[6+1] = 1;
\\ conf_initial
\\ print(conf_to_string(conf_initial));
for(i=1,#conf_initial_vector, conf_print(conf_initial_vector[i]));

\\ conf_permute(conf_initial,left_perm)
\\ conf_permute(conf_initial,right_perm)
\\ conf_print(conf_permute(conf_initial,left_perm));
\\ conf_print(conf_permute(conf_initial,right_perm));

\\-------------------------------------

conf_to_index_locked = 0;
conf_to_index_key   = List([]);
conf_to_index_value = List([]);
index_to_conf = List([]);
conf_to_index(conf) =
{
  my(pos=setsearch(conf_to_index_key,conf));
  if(pos>0, return(conf_to_index_value[pos]));

  if(conf_to_index_locked,
     error("Unknown conf ",conf_to_string(conf)));

  listput(index_to_conf,conf);
  my(index = #index_to_conf);

  pos = setsearch(conf_to_index_key,conf,1);  \\ where to insert
  listinsert(conf_to_index_key,  conf,  pos);
  listinsert(conf_to_index_value,index, pos);

  \\ when treating mirrors as equivalent insert also mirror image conf
  if(want_mirror,
     conf = conf_permute(conf,mirror_perm);
     pos = setsearch(conf_to_index_key,conf,1); \\ where to insert
     if(pos,
        listinsert(conf_to_index_key,  conf,  pos);
        listinsert(conf_to_index_value,index, pos)));

  index;
}

{
  for(i=1,#conf_initial_vector, conf_to_index(conf_initial_vector[i]));
  my(i=1);
  while(i <= #index_to_conf,
        my(conf=index_to_conf[i]);
        i++;
        for(p=1,#perms_vector,
           my(perm=perms_vector[p],
              new_conf=conf_permute(conf,perm));
           if(conf_is_empty(new_conf),
              next());
           conf_to_index(new_conf)));
}
conf_to_index_locked = 1;
index_to_conf = Vec(index_to_conf);

for(i=1,#index_to_conf, if(conf_is_empty(index_to_conf[i]),print("empty ",i);quit(1)));

size = #index_to_conf;
print("configurations size=",size);

{
m = matrix(size,size);
m_lr=matrix(size,2);
for(index=1,size,
   my(conf = index_to_conf[index]);
   for(p=1,#perms_vector,
      my(perm      = perms_vector[p],
         new_conf  = conf_permute(conf,perm));
      if(conf_is_empty(new_conf),
         print("skip ",conf_to_string(conf)," ("p") -> empty");
         next());
      my(new_index = conf_to_index(new_conf));
      m[new_index,index]++;
      m_lr[index,p] = new_index;
   ));
}

mat_sum(m,f) = my(s=matsize(m)); sum(i=1,s[1], sum(j=1,s[2], f(m[i,j])));
mat_count_nonzeros(m) = mat_sum(m,(x)->x!=0);
mat_count_equal(m,value) = mat_sum(m,(x)->x==value);

\\ print("m determinant ",matdet(m));
print("m total weight ", sum(c=1,size, vecsum(m[,c])));
\\ print("m non-zeros    ",mat_count_nonzeros(m));
\\ print("m twos         ",mat_count_equal(m,2));
initial = vectorv(size,i, i==1);

initial = vectorv(size,i, 0);
for(i=1,#conf_initial_vector, \
    initial[conf_to_index(conf_initial_vector[i])]++);
\\ print("initial "initial)

{
for(i=1,#index_to_conf, \
   my(conf = index_to_conf[i],
      conf_left = conf_permute(conf,left_perm));
   if(conf_left == conf_permute(conf,right_perm),
      print("conf ",conf_to_string(conf),
            " left,right = ",conf_to_string(conf_left))));
}

\\----------------------
\\ print conf descents

\\ {
\\ my(v=initial);
\\ for(k=0,5,
\\    print("confs k=" k);
\\    for(i=1,#v, if(v[i], conf_print(index_to_conf[i])));
\\    for(i=1,#v, if(v[i],
\\                   my(conf=index_to_conf[i]);
\\                   print(conf_to_string(conf),
\\                         " -> ",
\\                         conf_to_string(conf_permute(conf,left_perm)),
\\                         " "
\\                         conf_to_string(conf_permute(conf,right_perm)))));
\\    v = m*v);
\\ }

\\-----------------------------------------------------------------------------

{
  my(count=0);
  for(i=1,size,
     for(j=1,size,
        if(m[i,j]>1,
           print(i","j" m="m[i,j]);
           count++)));
  print("total " count " doubles");
}
{
  my(maxdepth = 5,
     reached  = matrix(maxdepth,size,depth,i, depth==1));
  for(depth=2,maxdepth,
     for(i=1,size,
        if(reached[depth-1,i],
           for(j=1,2,
              my(r=m_lr[i,j]); if(r, reached[depth,r] = 1)))));
  my(prev_unreached=0);
  for(depth=1,maxdepth,
     my(count_reached   = vecsum(reached[depth,]),
        count_unreached = size - count_reached,
        new_unreached   = count_unreached - prev_unreached);
     print("depth ",depth,
           " reached ",count_reached,
           " unreached ",count_unreached,
           " new ",new_unreached);
     for(i=1,size,
        if(depth>1 && reached[depth-1,i] && !reached[depth,i],
           print("  ",conf_to_string(index_to_conf[i]))));
     if(depth>1&&new_unreached==0, break());
     prev_unreached = count_unreached);
}

\\ {
\\   my(unseen=vector(size,i,1));
\\   for(i=1,size,
\\      my(p=m_lr[i,1]);
\\      if(!unseen[p], m_lr[i,1]=m_lr[i,2]; m_lr[i,2]=p); \\ swap
\\      unseen[m_lr[i,1]]=0);
\\   print("unseen ",vecsum(unseen));
\\ }
\\ {
\\   my(unseen=vector(size,i,1));
\\   for(i=1,size, unseen[m_lr[i,1]]=0; unseen[m_lr[i,2]]=0);
\\   print("unseen ",vecsum(unseen));
\\ }

\\-----------------------------------------------------------------------------

conf_index_single_destination(index) =
{
  if(m_lr[index,1]==0, m_lr[index,2],
     m_lr[index,2]==0, m_lr[index,1],
     0);
}
{
  print("single destinations");
  for(i=1,#index_to_conf,
     my(new_index = conf_index_single_destination(i));
     if(new_index,
        my(conf = index_to_conf[i],
           new_conf = index_to_conf[new_index],
           new_single = conf_index_single_destination(new_index));
        print(i "   ",conf_to_string(conf),
              " -> ",
              new_index,"  ",conf_to_string(new_conf)
              \\ , " [" new_single "]"
             )));
}
\\ count 18
\\ 7  -> 14 
\\ 11  -> 4  -> 13  -> 6  -> 15 
\\ 14  -> 5  -> 12  -> 3 
\\ 24  -> 32  -> 42  -> 20 
\\ 25  -> 33  -> 43  -> 21 
\\ 34  -> 44 
\\ 41  -> 57 
\\ 81  -> 105 
\\ 86  -> 111 

\\-----------------------------------------------------------------------------
rec_length = size+50;

\\ print("matsolve\n");
\\ mx = matid(size) - m*x;
\\ gm = matsolve(mx,initial);
\\ gm

print("\n\nfull");
full_row = vector(size,i, conf_is_full(index_to_conf[i])) / 2;
full_conf_count = vecsum(full_row);
print("full conf count ",full_conf_count," / ",size);
full = List([]);
{
  my(v=initial);
  for(k=0,rec_length,
     my(b=full_row*v);
     listput(full,b);
     if(k<20,print("k="k"  ",b));
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
full=Vec(full);
full[1..min(15,#full)]
\\ full=full[12..-1];
recurrence_guess(full)


