\\ Copyright 2014, 2015, 2016 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

default(recover,0);
allocatemem(300 * 10^6);
read("configurations.gp");
read("recurrence-guess.gp");
read("c-curve-defines.gp");

\\-----------------------------------------------------------------------------
rec_length = 2*size+50;
rec_length = 2*99+50;


\\-----------------------------------------------------------------------------
\\ configurations with same poly11 gf parts

poly_to_monic(p) = p / pollead(p);
{
  print("same poly11 parts");
  my(m = Map());
  for(i=1,#conf_occurring_vector,
     my(conf = conf_occurring_vector[i]);
     conf_count_visits(conf)==1 || conf_count_visits(conf)==3 || next();

     my(gf = conf_to_gf(conf));
     my(p = gf_poly11_part(gf));
     if(p==0, next());
     \\ print();
     \\ pol_ascending_print(p);
     p *= revpoly11(x);
     if(poldegree(p)>=11,
        print("gf ",gf_poly11_part(gf));
        print(p);
        quit());
     p = poly_to_monic(p);
     if(mapisdefined(m,p),
        my(l = mapget(m,p));
        listput(l,conf)
        ,
        my(l = List([]));
        listput(l,conf);
        mapput(m,p,l))
  );
  my(v = Vec(m));
  v = vecsort(v,cmp);
  print(#v " different numerators");
  for(i=1,#v,
     my(p = v[i]);
     my(l = mapget(m,p));
     print();
     print(p);
     for(j=1,#l,
        my(conf = l[j]);
        print("  ",conf_to_string(conf)));
  );
}
quit();


\\-----------------------------------------------------------------------------
\\ configuration counts sum with and without 9
\\ no cancellations

{
  for(i=1,#conf_occurring_vector,
     my(conf = conf_occurring_vector[i]);
     if(conf[9] != 1, next());
     my(other_conf = conf); other_conf[9] = 0;

     my(gf = conf_to_gf(conf),
        other_gf = conf_to_gf(conf),
        sum_gf = gf + other_gf);
     print(conf_to_string(conf),"  ",
           poldegree(gf_to_linear_recurrence_poly(sum_gf)));
  );
}
quit();


\\-----------------------------------------------------------------------------
\\ configurations which occur with or without 9

{
  print("with 9 but not without");
  my(count_with = 0);
  for(i=1,#conf_occurring_vector,
     my(conf = conf_occurring_vector[i]);
     if(conf[9] != 1, next());
     my(other_conf = conf); other_conf[9] = 0;
     if(conf_occurs(other_conf), next());
     conf_print(conf);
     count_with++;
  );

  print("without 9 but not with");
  my(count_without = 0);
  for(i=1,#conf_occurring_vector,
     my(conf = conf_occurring_vector[i]);
     if(conf[9] != 0, next());
     my(other_conf = conf); other_conf[9] = 1;
     if(conf_occurs(other_conf), next());
     conf_print(conf);
     count_without++;
  );
  print(count_with"  with 9 but not without");
  print(count_without"  without 9 but not with");
}
quit();


\\-----------------------------------------------------------------------------
\\ squares with any edge on sides
\\ 4-sides 300 / 753
\\ 0  1  2  3  4  5
\\ 0, 0, 0, 0, 0, 1, 2, 5, 10, 23, 50, 114, 250, 556, 1218
\\ poly8alt, poly9, poly11, poly14, poly18 and various

print("\n\nsquareany");
conf_count_squareany(conf) = {
  if(! (conf[1] || conf[2]), return(0));
    (conf[1] || conf[2])
  + (conf[7] || conf[8])
  + (conf[9] || ((conf[2]+conf[7]+conf[11])%2))
  + (conf[10] || conf[11]);
}
conf_count_squareany_sides(conf) = {
  [ (conf[1] || conf[2]),
    (conf[7] || conf[8]),
    (conf[9] || ((conf[2]+conf[7]+conf[11])%2)),
    (conf[10] || conf[11]) ];
}
conf_is_squareany(conf) = \
  conf_count_squareany(conf) == 2;
squareany_row = apply(conf_is_squareany, index_to_conf);
squareany_conf_count = vecsum(squareany_row);
print("squareany conf count ",squareany_conf_count," / ",size);

gQ1(x) = conf_predicate_n_to_gf(conf_count_squareany,1);
gQ2(x) = conf_predicate_n_to_gf(conf_count_squareany,2);
gQ3(x) = conf_predicate_n_to_gf(conf_count_squareany,3);
gQ4(x) = conf_predicate_n_to_gf(conf_count_squareany,4);

factor(gf_to_linear_recurrence_poly(gQ1(x)))
factor(gf_to_linear_recurrence_poly(gQ2(x)))
factor(gf_to_linear_recurrence_poly(gQ3(x)))
factor(gf_to_linear_recurrence_poly(gQ4(x)))
gf_to_linear_recurrence_poly(gQ1(x)) == gf_to_linear_recurrence_poly(gQ2(x))
gf_to_linear_recurrence_poly(gQ1(x)) == gf_to_linear_recurrence_poly(gQ3(x))
gf_to_linear_recurrence_poly(gQ1(x)) == gf_to_linear_recurrence_poly(gQ4(x)) / (x-2)

{
  for(i=1,#index_to_conf,
     my(conf=index_to_conf[i]);
     if(!conf_is_squareany(conf),next());
     my(gf=conf_index_to_gf[i]);
     my(count=gf_term(gf,0));
     if(count,
        print(count,"  ",conf_count_squareany_sides(conf));
        conf_print(conf));
  );
\\ quit();
}

squareany = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=squareany_row*v);
     listput(squareany,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
squareany=Vec(squareany);
squareany[1..15]
\\ squareany=squareany[12..-1];
recurrence_guess(squareany)
quit();


\\-----------------------------------------------------------------------------
\\ edges with vertex degrees 1 to 4 at each end
\\ 1,1 = 90 terms
\\ 4,4 = 2/753 = 91 terms  with or without triangle 9

print("\n\nedges");
conf_degree_left(conf) = {
  max(conf[1] + conf[3] + conf[5] + conf[7], \\ enter
      conf[2] + conf[4] + conf[6] + conf[8]); \\ leave
}
conf_degree_right(conf) = {
  max(conf[1] + conf[11] + conf[13] + conf[15], \\ enter
      conf[2] + conf[10] + conf[12] + conf[14]); \\ leave
}
conf_count_edgeends(conf) = conf_degree_left(conf) + 10*conf_degree_right(conf);
conf_is_edgeends(conf) = \
  (conf[1] || conf[2]) && conf_count_edgeends(conf) == 13;
edgeends_row = apply(conf_is_edgeends, index_to_conf);
edgeends_conf_count = vecsum(edgeends_row);
print("edgeends conf count ",edgeends_conf_count," / ",size);

edgeends = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=edgeends_row*v);
     listput(edgeends,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
edgeends=Vec(edgeends);
edgeends[1..15]
\\ edgeends=edgeends[12..-1];
recurrence_guess(edgeends)
quit();


\\-----------------------------------------------------------------------------
\\ threeside triangle with all 3 sides of triangle filled  len=65
\\
\\ all
\\ 2^k, poly8alt, poly9, poly11, poly18           sans poly8, poly14
\\ recurrence len=65
\\
\\ 2-sides
\\ len=64
\\
\\ 1-sides
\\ len=64
\\ [0, 0, 0, 2, 6, 14, 26, 54, 114, 238, 476, 960, 1922, 3850, 7636]

print("\n\nthreeside");
conf_count_three(conf) = conf[2] + conf[8] + conf[10];
conf_is_threeside(conf) = conf[1] && conf_count_three(conf) == 1;
threeside_row = apply(conf_is_threeside, index_to_conf);
threeside_conf_count = vecsum(threeside_row);
print("threeside conf count ",threeside_conf_count," / ",size);

threeside = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=threeside_row*v);
     listput(threeside,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
threeside=Vec(threeside);
threeside[1..15]
\\ threeside=threeside[12..-1];
recurrence_guess(threeside)
quit();


\\-----------------------------------------------------------------------------
\\ all conf triangle counts go to increase and decrease

{
  my(ft = matrix(16,16));
  for(index=1,#index_to_conf,
     my(conf = index_to_conf[index],
        count = vecsum(conf));

     ft[1+count, 1+vecsum(conf_permute(conf,left_perm))]++;
     ft[1+count, 1+vecsum(conf_permute(conf,right_perm))]++);

  for(i=1,16,
     print1(i-1,"  ");
     for(j=1,16,
        if(ft[i,j], print1(" ",j-1)));
     print())
}
quit();


\\-----------------------------------------------------------------------------
\\  odd .--->. odd     turn(n) = 1 - CountLowZeros(n)
\\     |^    |^        180 is turn(n) == 2 mod 4
\\     ||    ||        is CountLowZeros(n) == 3 mod 4
\\     v|    v|        so n-1 is odd so turn(n)=+1 = left
\\      .    .         so n+1 is odd so turn(n)=+1 = left
\\   even   even
\\             
\\             
\\      .     .

\\ validate also 1,2 with nothing left means must have 14 in, 10 out

conf_validate_more(conf) =
{
  if(! conf_validate_enter_leave(conf), return(0));
  if(conf[1] && conf[2],
     if(conf_count_left_enter(conf) == 1,
        if(! (conf[14] && conf[10]), return(0)));
     if(conf_count_right_enter(conf) == 1,
        if(! (conf[4] && conf[8]), return(0))));
  return(1);
}
confs_count_sample
sum(n=1,2^15-1, my(conf=binary_padded(n,15)); conf_validate_more(conf))
quit();


\\-----------------------------------------------------------------------------
\\ how many 7,8 and 10,11 pairings
\\ 32 with minimum 7 segments

{
  my(count=0, segs=15);
  for(index=1,#index_to_conf,
     my(conf=index_to_conf[index]);
     if(conf[7] && conf[8] && conf[10] && conf[11],
        count++;
        segs = min(segs,vecsum(conf));
        if(! conf[9], conf_print(conf));
       );
  );
  print("count ",count," min segs ",segs);
}
quit();


\\-----------------------------------------------------------------------------
\\ configurations valid by enter/leave count but which don't occur

read("c-curve-defines.gp");
{
for(n=1,2^15-1,
   my(conf=binary_padded(n,15));
   if(! conf_validate(conf), next());
   if(conf_to_index_maybe(conf) >= 0, next());
   conf_print(conf);
);
}
quit();


\\-----------------------------------------------------------------------------
\\ two opposing 8,10 not 1,9  len=17 poly9, poly8alt
\\ angle two 1,8 not 9,10 len=103
\\ angle two 1,8 not 9,10 or other way len=17 poly9, poly8alt
\\           same when all 3 ways
\\ 8 or 10   poly9
\\ 8 or 10 and not both    poly9

print("\n\ntwoopposite");
conf_is_twoopposite(conf) = conf[8] && conf[10] && ! conf[1] && ! conf[9];
conf_is_twoopposite(conf) = (conf[1] && conf[10] && ! conf[8] && ! conf[9]) \
                         || (conf[1] && conf[8] && ! conf[10] && ! conf[9]) \
                         || (conf[8] && conf[10] && ! conf[1] && ! conf[9]);
conf_is_twoopposite(conf) = (conf[8] + conf[10]) >= 1;
twoopposite_row = apply(conf_is_twoopposite, index_to_conf);
twoopposite_conf_count = vecsum(twoopposite_row);
print("twoopposite conf count ",twoopposite_conf_count," / ",size);

twoopposite = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=twoopposite_row*v);
     listput(twoopposite,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
twoopposite=Vec(twoopposite);
twoopposite[1..15]
\\ twoopposite=twoopposite[12..-1];
recurrence_guess(twoopposite)
quit();


\\-----------------------------------------------------------------------------
\\ nine poly9
\\ nine and 2 len=30 poly9, poly18

print("\n\nnine");
conf_is_nine(conf) = conf[1] && conf[9] && conf[2];
nine_row = apply(conf_is_nine, index_to_conf);
nine_conf_count = vecsum(nine_row);
print("nine conf count ",nine_conf_count," / ",size);

nine = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=nine_row*v);
     listput(nine,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
nine=Vec(nine);
nine[1..15]
\\ nine=nine[12..-1];
recurrence_guess(nine)
quit();


\\-----------------------------------------------------------------------------
\\ len=31 poly9, poly18
\\ five under and over len=93

print("\n\nthreeunder");
conf_is_threeunder(conf) = conf[1] && conf[5] && conf[13] \
                          && conf[6] && conf[12];
threeunder_row = apply(conf_is_threeunder, index_to_conf);
threeunder_conf_count = vecsum(threeunder_row);
print("threeunder conf count ",threeunder_conf_count," / ",size);

threeunder = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=threeunder_row*v);
     listput(threeunder,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
threeunder=Vec(threeunder);
threeunder[1..15]
\\ threeunder=threeunder[12..-1];
recurrence_guess(threeunder)
quit();


\\-----------------------------------------------------------------------------
\\ len=65 poly8alt, poly9, poly11, poly18

print("\n\ntwodoubles");
conf_is_twodoubles(conf) = conf[7] && conf[8] && conf[10] && conf[11];
twodoubles_row = apply(conf_is_twodoubles, index_to_conf);
twodoubles_conf_count = vecsum(twodoubles_row);
print("twodoubles conf count ",twodoubles_conf_count," / ",size);

twodoubles = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=twodoubles_row*v);
     listput(twodoubles,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
twodoubles=Vec(twodoubles);
twodoubles[1..15]
\\ twodoubles=twodoubles[12..-1];
recurrence_guess(twodoubles)
quit();


\\-----------------------------------------------------------------------------
\\ len=31 poly9, poly18
\\ or 1=unfilled is len=30

print("\n\nthreeline");
conf_is_threeline(conf) = conf[6] && conf[1] && conf[12];
threeline_row = apply(conf_is_threeline, index_to_conf);
threeline_conf_count = vecsum(threeline_row);
print("threeline conf count ",threeline_conf_count," / ",size);

threeline = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=threeline_row*v);
     listput(threeline,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
threeline=Vec(threeline);
threeline[1..15]
\\ threeline=threeline[12..-1];
recurrence_guess(threeline)
quit();


\\-----------------------------------------------------------------------------

print("\n\ndouble_seg");
double_seg_row = vector(size,i, conf_is_double_seg(index_to_conf[i])) / 2;
double_conf_count = vecsum(double_seg_row);
print("double conf count ",double_conf_count," / ",size);
double_seg = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=double_seg_row*v);
     listput(double_seg,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
double_seg=Vec(double_seg);
double_seg[1..15]
\\ double_seg=double_seg[12..-1];
recurrence_guess(double_seg)

\\-----------------------------------------------------------------------------
\\ of one configuration, eg. conf_all_ones fully enclosed
print("one configuration");

\\ conf_first_double
{for(index=1,size,
    my(conf=index_to_conf[index]);
    if(conf_is_double_seg(conf), conf_first_double=conf; break()));}
\\ conf_all_ones

{
  my(conf = conf_first_double,
     index = conf_to_index(conf));
  \\ my(index = 50,
  \\    conf = index_to_conf[index]);
  print("\nconf number ",index," = ",conf_to_string(conf));
  conf_print(conf);
  my(index_seg_row = vector(size,i, i==index));
  my(index_seg = List([]));
  print("rec_length ", rec_length);

  my(v=initial);
  for(i=0,rec_length,
     my(b=index_seg_row*v);
     listput(index_seg,b);
       \\ print(b," of ",vecsum(v));
     v = m*v);

  index_seg=Vec(index_seg);
  print("values ",index_seg[1..30], "...");
  \\ index_seg=index_seg[12..-1];
  recurrence_guess(index_seg);
  quit();
}

\\-----------------------------------------------------------------------------

print("\n\nsingle_seg");
single_seg_row = apply(conf_is_single_seg_34, index_to_conf);
single_conf_count = vecsum(single_seg_row);
print("single conf count ",single_conf_count," / ",size);

single_seg = List([]);
{
  my(v=initial);
  for(i=0,rec_length,
     my(b=single_seg_row*v);
     listput(single_seg,b);
     \\ print(b," of ",vecsum(v));
     v = m*v);
}
single_seg=Vec(single_seg);
single_seg[1..15]
\\ single_seg=single_seg[12..-1];
recurrence_guess(single_seg)
quit();


\\----------------------

conf_count_visits_left(conf) = conf[1] + conf[7] + conf[5] + conf[3];
conf_count_visits_right(conf) = conf[12] + conf[10] + conf[2] + conf[14];
{
num_visits_row = vector(4,n,
                        vector(size,i,
                               (conf_count_visits_left(index_to_conf[i])==n)
                               + (conf_count_visits_right(index_to_conf[i])==n)));
}

print("\npoint visits ==n");
{
visits_matrix = matrix(4,rec_length+1);
for(num_visits=1,4,
   print("num_visits=",num_visits);
   my(count = List([]));
   my(row = num_visits_row[num_visits],
      v = initial);
   for(i=0,rec_length,
      my(b=row*v);
      b /= 4;   \\ 4 orientations
      b /= 2;   \\ both ends
      if(num_visits==1, b++);  \\ final point of single-visits
      listput(count,b);
      \\ print(b," of ",vecsum(v));
      v = m*v);
   count=Vec(count);
   visits_matrix[num_visits,] = count;
   print(count[1..35]);
   print(count[36..50]);
\\   count=count[12..-1];
   recurrence_guess(count)
);
}
print("distinct points");
{
  my(count = sum(i=1,4, visits_matrix[i,]));
  count=Vec(count);
  print(count[1..20]);
  recurrence_guess(count)
}

\\----------------------

print("\nsquare sides");
num_sides_row = vector(4,n, \
                        vector(size,i, conf_count_sides(index_to_conf[i]) == n));
{
for(num_sides=1,4,
   print("num_sides="num_sides);
   count = List([]);
   my(row = num_sides_row[num_sides],
      v = initial);
   print("configurations " vecsum(row));
   for(i=0,rec_length,
      my(b=row*v);
      b /= 4;   \\ 4 orientations
      listput(count,b);
      \\ print(b," of ",vecsum(v));
      v = m*v);
   count=Vec(count);
   print(count[1..15]);
\\   count=count[12..-1];
   recurrence_guess(count)
);
}
print("total of sides ",vecsum(vecsum(num_sides_row))/size)



\\-----------------------------------------------------------------------------
quit
read("pol-pfrac.gp")

{
for(num_visits=1,4,
   print("\nvisits=",num_visits);
   g = num_visits_row[num_visits]*gm;
   print("  ",g);
   p = pol_partial_fractions(g);
   for(i=1,#p, print("  ",str_poly_ascending(p[i])));
);
}
{
for(num_sides=1,4,
   print("\nsides=",num_sides);
   g = num_sides_row[num_sides]*gm;
   print("  ",g);
   p = pol_partial_fractions(g);
   for(i=1,#p, print("  ",str_poly_ascending(p[i])));
);
}

print("\ndouble segments");
gDouble_seg = double_seg_row*gm
gDouble_seg == 1/(1-2*x) - gSingle_seg
{
   my(p = pol_partial_fractions(gDouble_seg));
   for(i=1,#p, print("  ",str_poly_ascending(p[i])));
}

print("\nsingle segments");
gSingle_seg = single_seg_row*gm
{
   my(p = pol_partial_fractions(gSingle_seg));
   for(i=1,#p, print("  ",str_poly_ascending(p[i])));
}

