\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.

default(strictargs,1);
read("c-curve-defines.gp");

check_k(func1,func2) =
{
  for(k=3,20,
     my(got = func1(k));
     my(want = func2(k));
     if(! nearly_equal(got, want),
        print("wrong at k="k);
        print("got  "got);
        print("want "want);
        quit(1)));
  print("ok");
}

check(func1,func2) =
{
  for(k=0,20,
     for(n=5,20,
        my(f=n/25, t=Pi*f);
        if(! nearly_equal(func1(k,t), func2(k,t), 1e-11),
           print("wrong at k="k" n="n);
           quit(1))));
  print("ok");
}


differ(x,y) =
{
  \\ if(type(x)!=type(y),
  \\    return(Strprintf("different types %s %s",type(x),type(y))));
  if(type(x)!="t_VEC",
     if(! nearly_equal(x,y),
        return(Strprintf("different values\n%s\n%s",x,y)));
     return(0));
  if(length(x)!=length(y),
     return(Strprintf("different lengths %s %s",length(x),length(y))));
  for(i=1,#x,
     my(d=differ(x[i],y[i]));
     if(d != 0, return(Strprintf("at index %d %s",i,d))));
  return(0);
}

check_equal(got,want,name="") =
{
  my(d = differ(got,want));
  if(d != 0,
     print("wrong ",name,"\n",d); quit(1),
     print("ok ", name));
}

\\-----------------------------------------------------------------------------

\\ G_theta(k,t) = 1/2*cos(t/2)*sin(t/2)^(k-1)*(2^k-1);
\\ b_theta_abs(t) = 2*abs(sin(t/2));

right=Pi/2;
nearly_equal(b_theta_abs(right), sqrt2) || error();
my(k=10); nearly_equal(G(k), G_theta(k,right)) || error();

\\ cf I_t(k) = G(k) *1/sqrt2 + 1/2*sqrt2^k *1/sqrt2;
I_t_theta_of_k_minus_1(k,t) = G_theta(k-1,t) * cos(t/2) \
                              + 1/4 * b_theta_abs(t)^k;
I_t_theta_ref(k,t) = I_t_theta_of_k_minus_1(k+1,t);
check_k((k)->I_t_theta_ref(k,right), I_t)
check_equal(I_t_theta_ref(0,Pi),   1/2, "I_t_theta_ref 0,Pi")
check_equal(I_t_theta_ref(0,Pi/2), 1/4*sqrt(2))

print("I_t_ref(90) ",vector(10,k, I_t_theta_ref(k,right)))
print("I_t         ",vector(10,k, I_t(k)*1.0))

\\ this form by thinking of half way along the k-1 sub-part
I_t_theta(k,t) = G_theta(k,t)*cos(t/2)  +  1/2*b_theta_abs(t)^k*sin(t/2);
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  1/2*(    sin(t/2)^(k-1)*(2^k - 1)  * cos(t/2)^2
       + (2*sin(t/2))^k              *  sin(t/2) );
  ;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  if(t==0, return(0));
  1/2*(  sin(t/2)^k*(2^k - 1)  * 1/sin(t/2) * (1 - sin(t/2)^2)
       + (2*sin(t/2))^k              *  sin(t/2) );
  ;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  if(t==0, return(0));
  1/2*(  (2*sin(t/2))^k  * 1/sin(t/2) * (1 - sin(t/2)^2) 
       - sin(t/2)^k  * 1/sin(t/2) * (1 - sin(t/2)^2) 
       + (2*sin(t/2))^k              *  sin(t/2) );
  ;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  if(t==0, return(0));
  1/2*(  (2*sin(t/2))^k  * ( 1/sin(t/2) * (1 - sin(t/2)^2) 
                            +  sin(t/2) )
       - sin(t/2)^k  * 1/sin(t/2) * (1 - sin(t/2)^2) 
      )
  ;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  if(t==0, return(0));
  1/2 * 1/sin(t/2) *( (2*sin(t/2))^k - sin(t/2)^k *cos(t/2)^2 )
;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta(k,t) =
{
  if(t==0, return(0));
  1/2*( 2^k - cos(t/2)^2 ) * sin(t/2)^(k-1)
;
}
check(I_t_theta_ref,I_t_theta);

I_t_theta_squared_ref(k,t) = I_t_theta_ref(k,t)^2;

I_t_theta_squared(k,t) =
{
  if(t==0, return(0));
  1/4 * 1/sin(t/2)^2 *( (2*sin(t/2))^k - sin(t/2)^k *cos(t/2)^2 )^2
;
}
check(I_t_theta_squared,I_t_theta_squared_ref);
check_k((k)->I_t_theta_squared(k,right), (k)->I_t(k)^2)

\\-----------------------------------------------------------------------------
\\ Iz

\\ Iz(k,t) = 2*Iz(k-1,t) + 2*t(k-1,t)^2
\\ Iz(k,t) = sum j=0,k-1  of  2^j * 2*t(k-1-j,t)^2

I_z_theta_ref(k,t) = if(k==0,0, 2*I_z_theta_ref(k-1,t) + 2*2^(k-1)*I_t_theta_ref(k-1,t)^2);
I_z_theta_ref = memoize(I_z_theta_ref);
check_k((k)->I_z_theta_ref(k,right), Iz)

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  2^k * sum(j=0,k-1, I_t_theta_ref(k-j-1,t)^2 );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  sum(j=0,k-1,
        2^j *
        2*2^(k-j-1)*I_t_theta_ref(k-j-1,t)^2
  );
}
check(I_z_theta_ref,I_z_theta);
print("pi   ", I_z_theta(10,Pi));
print("zero ", I_z_theta(10,0));

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  sum(j=0,k-1,
        2^j * 2*2^(k-j-1)*
        1/4 * 1/sin(t/2)^2 *( (2*sin(t/2))^(k-1-j) - sin(t/2)^(k-1-j) *cos(t/2)^2 )^2
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  sum(j=0,k-1,
        2^j * 2*2^(k-j-1)*
        1/4 * 1/sin(t/2)^2 *(
                             (4*sin(t/2)^2)^(k-1-j)
                             - 2*sin(t/2)^(k-1-j) *cos(t/2)^2
                                * (2*sin(t/2))^(k-1-j)
                             + (sin(t/2)^2)^(k-1-j) *cos(t/2)^4
                            )
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  1/4 * 1/sin(t/2)^2 *
  sum(j=0,k-1,
        2^j * 2*2^(k-j-1)*
        (
         (4*sin(t/2)^2)^(k-1-j)
         - 2*sin(t/2)^(k-1-j) *cos(t/2)^2 * (2*sin(t/2))^(k-1-j)
         + (sin(t/2)^2)^(k-1-j) *cos(t/2)^4
        )
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  1/2 * 1/sin(t/2)^2 *
  sum(j=0,k-1,
        2^j *
        (
                         (8*sin(t/2)^2)^(k-1-j)
         - 2*cos(t/2)^2* (4*sin(t/2)^2)^(k-1-j)  
         + cos(t/2)^4*   (2*sin(t/2)^2)^(k-1-j) 
        )
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  1/2 * 1/sin(t/2)^2 *
  sum(j=0,k-1,
        2^j *
        (
                         (8*sin(t/2)^2)^(k-1-j)
         - 2*cos(t/2)^2* (4*sin(t/2)^2)^(k-1-j)  
         + cos(t/2)^4*   (2*sin(t/2)^2)^(k-1-j) 
        )
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  if(t == 0, return(0));
  1/2 * 1/sin(t/2)^2 *
  (
   if( (8*sin(t/2)^2) == 2, k*2^(k-1),
      (2^k - (8*sin(t/2)^2)^k) / (2 - 8*sin(t/2)^2) )
   
   - 2*cos(t/2)^2*
   if( (4*sin(t/2)^2) == 2, k*2^(k-1),
      (2^k - (4*sin(t/2)^2)^k) / (2 - 4*sin(t/2)^2) )
   
   + cos(t/2)^4*
   if( (2*sin(t/2)^2) == 2, k*2^(k-1),
       (2^k - (2*sin(t/2)^2)^k) / (2 - 2*sin(t/2)^2) )
  );
}
check(I_z_theta_ref,I_z_theta);

my(t=Pi/3); nearly_equal(4*sin(t/2)^2, 1) || error();
my(t=Pi/2); nearly_equal(2*sin(t/2)^2, 1) || error();
my(t=Pi);   nearly_equal(  sin(t/2)^2, 1) || error();

I_z_theta_pi_on_3(k) =
{
  (k*2^k
   - 3 * (2^k - 1) 
   + 3/4 * (2^k - (1/2)^k) );
}
I_z_theta_pi_on_3(k) = (k-9/4)*2^k + 3 - 3/4 * (1/2)^k;
my(k=10); print("pi/3  ",I_z_theta_pi_on_3(k)*1.0);
my(k=10); print("pi/3  ",I_z_theta(k,Pi/3*.99999));
my(k=10); nearly_equal(I_z_theta_pi_on_3(k), I_z_theta(k,Pi/3*(1-1e-30)), 1e-3)

I_z_theta_pi(k) = 1/12 * 2^k * (4^k - 1);
my(k=10); print("pi  ",I_z_theta_pi(k)*1.0);
my(k=10); print("pi  ",I_z_theta(k,Pi*.99999));
my(k=10); nearly_equal(I_z_theta_pi(k), I_z_theta(k,Pi*(1-1e-10)), 1e-3)

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

  1/2 * 1/sin(t/2)^2 * 2^k *
  (
   1/2 / (1 - 4*sin(t/2)^2)             * (1 - (4*sin(t/2)^2)^k)
   - cos(t/2)^2 / (1 - 2*sin(t/2)^2)    * (1 - (2*sin(t/2)^2)^k)
   + 1/2*cos(t/2)^2                     * (1 - (  sin(t/2)^2)^k)
  )
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

  1/2 * 1/sin(t/2)^2 * 2^k *
  (
   1/2 / (4*sin(t/2)^2 - 1)             * ((4*sin(t/2)^2)^k - 1)
   - cos(t/2)^2 / (2*sin(t/2)^2 - 1)    * ((2*sin(t/2)^2)^k - 1)
   - 1/2*cos(t/2)^2                     * ((  sin(t/2)^2)^k - 1)
  )
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

  1/4 * 2^k   / cos(t) / (1 - 2*cos(t)) / (1 - cos(t)) *
  (
     ((4*sin(t/2)^2)^k - 1) * 2 * cos(t)
   + ((2*sin(t/2)^2)^k - 1) * 2 *          (1 - 2*cos(t)) * (1 + cos(t))
   - ((  sin(t/2)^2)^k - 1) *     cos(t) * (1 - 2*cos(t)) * (1 + cos(t))
  )
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

  1/4 * 2^k *
  ( 
     ((4*sin(t/2)^2)^k - 1) * 2 / (1 - 2*cos(t)) / (1 - cos(t))
   + ((2*sin(t/2)^2)^k - 1) * 2 / cos(t) / tan(t/2)^2
   - ((sin(t/2)^2)^k - 1)   / tan(t/2)^2
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

  1/4 * 2^k *
  ( 
     ((4*sin(t/2)^2)^k) * 2 / (1 - 2*cos(t)) / (1 - cos(t))
   + ((2*sin(t/2)^2)^k) * 2 / cos(t) / tan(t/2)^2
   - (sin(t/2)^2)^k   / tan(t/2)^2
   - (2 - cos(t) ) * (1+cos(t)) / cos(t) / (1 - cos(t))
   -  2 / (1 - 2*cos(t))                 / (1 - cos(t))
  );
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

     1/2 * ((8*sin(t/2)^2)^k) / (1 - 2*cos(t)) / (1 - cos(t))
   + 1/2 * ((4*sin(t/2)^2)^k) / cos(t) / tan(t/2)^2
   - 1/4 * (2*sin(t/2)^2)^k   / tan(t/2)^2
   + 1/4 * 2^k * (2*cos(t)^2 - cos(t) - 2) / (1 - 2*cos(t)) / cos(t)  
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

     2 / (1 - 2*cos(t))          * (8*sin(t/2)^2)^(k-1)
   + (1 + cos(t)) / cos(t)       * (4*sin(t/2)^2)^(k-1)
   - 1/4 * (1 + cos(t))          * (2*sin(t/2)^2)^(k-1)    
   + 1/4 * (2*cos(t)^2 - cos(t) - 2) / (1 - 2*cos(t)) / cos(t)    * 2^k
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

     2 / (1 - 2*cos(t))          * (8*sin(t/2)^2)^(k-1)
   + (1 + cos(t)) / cos(t)       * (4*sin(t/2)^2)^(k-1)
   - 1/4 * (1 + cos(t))          * (2*sin(t/2)^2)^(k-1)    
   - 1/4 * (2 + ((1-2*cos(t))*cos(t))) / ((1-2*cos(t))*cos(t))   * 2^k
  ;
}
check(I_z_theta_ref,I_z_theta);

I_z_theta(k,t) =
{
  \\ if(t == 0, return(0));
  \\ if(t == Pi/2, return(Iz(k)));
  \\ if(t == Pi/3, return(I_z_theta_pi_on_3(k)));

     2 / (1 - 2*cos(t))          * (8*sin(t/2)^2)^(k-1)
   + (1 + cos(t)) / cos(t)       * (4*sin(t/2)^2)^(k-1)
   - 1/4 * (1 + cos(t))          * (2*sin(t/2)^2)^(k-1)    
   - 1/4 * (1 + 2/((1-2*cos(t))*cos(t)))   * 2^k
  ;
}
check(I_z_theta_ref,I_z_theta);

print("k=1 Pi/4");
I_z_theta_ref(1,Pi/4)
I_z_theta(1,Pi/4)
quit

\\ ( (2 - C ) * (1 + C)*(1 - 2*C) +  2 *C) == 2*C^3 - 3*C^2 - C + 2

\\  (1 - 2*cos(t)) * (1 - cos(t))
\\ =  2*C^2 - 3*C + 1
\\ = 2 - 2*S^2 - 3*C + 1
\\ = C^2 + 1 - S^2 - 3*C + 1 = C^2 - S^2 - 3*C + 2

\\ cos(t) = cos(t/2)^2

\\ -C^3 + 3/2*C^2 + 1/2*C - 1 == (2*C^2 - C - 2)*(C-1)
{
     ( - 1) * C
   + ( - 1) * (1 - 2*C) * (1 + C)
   - ( - 1) * C * 1/2*(1 + C) * (1 - 2*C)
}
1-4*-2*2 == 17
factor((-C^3 + 3/2*C^2 + 1/2*C - 1)/(C-1))
polroots(-C^3 + 3/2*C^2 + 1/2*C - 1)