\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  If not, see <http://www.gnu.org/licenses/>.

\\ Iline = 1/12 m L^2
\\ b = 90 - t/2
\\ cos(b) = 1/2 / (h/2) = 1/h
\\ h = 1/cos(b)
\\ when t=90 htve b=45 h=1/2*sqrt(2)
\\ r / (h/2) = sin(t/2)
\\ r = h/2 sin(t/2)
\\   = 1/2 1/cos(90-t/2) sin(t/2)
\\ r = 1/4
\\ I = 2*1/12*1/2*h^2 + 1/2*(1/4)^2
\\   = 1/12*h^2 + 1/32
\\   = 1/12*1/4*2 + 1/32 = 7/96 = 7/8*1/12
\\
\\                         m
\\ Iz(k) = 2*Iz(k-1) + 2*2^(k-1)*r^2
\\ r = sqrt(2)^(k-1) / sqrt(2) + G/sqrt(2)
\\   = sqrt(2)^(k-1) / sqrt(2) + G/sqrt(2)


read("../tools/memoize.gp");
b = 1 + I;
sqrt2 = quadgen(8);
vector_modulo(v,i) = v[(i% #v)+1];

nearly_equal(x,y) = abs(x-y) <= max(abs(x),max(abs(y),1))*1e-10;

check_count = 0;
check_exact(got,want, name="") =
{
  check_count++;
  if(got!= want,
     print("\nwrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count));
}

c_curve_segments(k) = {
  my(v = [1/2], end=1);
  for(i=1,k,
      v = concat(v, apply((z)->end+z*I, v));
      end *= b);
  v;
}

G_fractal(k) = 1/2*(1 - 1/2^k);
G(k) = G_fractal(k)*sqrt2^k;

\\-----------------------------------------------------------------------------

Iz_by_segments(k) =
{
  my(v = c_curve_segments(k),
     g = vecsum(v) / #v);
  \\ print("g="g);
  \\ print(norm(v[i]-g))
  sum(i=1,#v, norm(v[i]-g)) + 1/12*#v;
}
Iz_by_segments = memoize(Iz_by_segments);

Iz_points_by_segments(k) =
{
  my(v = c_curve_segments(k),
     g = vecsum(v) / #v);
  sum(i=1,#v, norm(v[i]-g));
}
Iz_points_by_segments = memoize(Iz_points_by_segments);

Iz(k) =
{
  if(k==0,return(1/12));
  my(l = sqrt2^(k-1),
     r = G(k-1)/sqrt2 + l*1/2/sqrt2);
  \\ print("r="r);
  2*Iz(k-1) + 2*2^(k-1)*r^2;
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

vector(10,k, Iz_by_segments(k-1))
vector(10,k, Iz(k-1))*12

check_exact(Iz(1), 2*1/12 + 2*1*(1/4*sqrt2)^2)


Iz(k) =
{
  if(k==0,return(1/12));
  2*Iz(k-1) + 2^k*(1/2*(sqrt2^(k-1) - 1/sqrt2^(k-1))/sqrt2
                   + sqrt2^(k-1)*1/2/sqrt2)^2;
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))


Iz(k) =
{
  if(k==0,return(1/12));
  2*Iz(k-1) + 2^k*(  1/2*sqrt2 * sqrt2^(k-1)
                   - 1/4*sqrt2 * 1/sqrt2^(k-1)
                  )^2;
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))


Iz(k) =
{
  if(k==0,return(1/12));
  2*Iz(k-1) + 2^k*(  (1/2*sqrt2 * sqrt2^(k-1))^2
                   - 2* 1/2*sqrt2 * sqrt2^(k-1) * 1/4*sqrt2 * 1/sqrt2^(k-1)
                   + (1/4*sqrt2 * 1/sqrt2^(k-1))^2
                  );
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

Iz(k) =
{
  if(k==0,return(1/12));
  2*Iz(k-1) + 2^k*(  1/2 * 2^(k-1)
                   - 1/2
                   + 1/8 * 1/2^(k-1)
                  );
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

Iz(k) =
{
  if(k==0,return(1/12));
  2*Iz(k-1) +  4^(k-1) - 2^(k-1) + 1/4;
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

Iz(k) =
{
  1/12*2^k + sum(j=0,k-1, 2^j * (4^(k-1-j) - 2^(k-1-j) + 1/4));
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))


Iz(k) =
{
  1/12*2^k + 1/4*(2^k-1) - k*2^(k-1) + (4^k - 2^k)/(4 - 2);
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

Iz(k) =
{
  1/12*2^k + 1/4*2^k - 1/4 - 1/2*k*2^k + 1/2*4^k - 1/2*2^k;
}
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))

Iz(k) =  1/2*4^k - (1/2*k + 1/6)*2^k - 1/4;
check_exact(vector(13,k, Iz(k-1)), vector(13,k, Iz_by_segments(k-1)))


Iz_points(k) =  Iz(k) - 1/12*2^k;
vector(10,k, Iz_points(k-1))*4
check_exact(vector(13,k, Iz_points(k-1)), vector(13,k, Iz_points_by_segments(k-1)))

Iz_points(k) =  1/2*4^k - (1/2*k + 1/4)*2^k - 1/4;
check_exact(vector(13,k, Iz_points(k-1)), vector(13,k, Iz_points_by_segments(k-1)))


