#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'chamfer','transform_to_horizontal','xy_rotate_minus90',
  'PartN','TopMiddlePost';
use MyPrintwrap ':all';


my $level = $ARGV[0] || 6;

print "    % generated by tools/$FindBin::Script $level\n";
my $path = Math::PlanePath::CCurve->new;

sub is_touch {
  my ($n, $n_hi) = @_;
  my ($x,$y) = $path->n_to_xy($n);
  foreach my $n_other ($path->xy_to_n_list($x,$y)) {
    next if $n_other == $n;
    next if $n_other > $n_hi;
    return 1;
  }
  return 0;
}

sub is_overlap {
  my ($n, $n_hi) = @_;
  my $n_other = $path->xyxy_to_n($path->n_to_xy($n+1), $path->n_to_xy($n));
  return defined($n_other) && $n_other < $n_hi;
}

my $n_start = PartN($level);
my $n_end = TopMiddlePost($level);
if ($level == 7) {
  $n_start -= ($n_start % 2);
}

my $transform = sub {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
};

my $n_hi = $n_end;
my ($x_end,$y_end) = $path->n_to_xy($n_end);

foreach my $n ($n_start .. $n_end-1) {
  my ($sx,$sy) = $path->n_to_xy($n);
  my ($tx,$ty) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my $arrow = 'my arrow';

  $sx -= $x_end;
  $sy -= $y_end;
  $tx -= $x_end;
  $ty -= $y_end;
  ($sx,$sy) = $transform->($sx,$sy);
  ($tx,$ty) = $transform->($tx,$ty);
  ($dx,$dy) = $transform->($dx,$dy);

  my $sdx = 0;
  my $sdy = 0;
  my $tdx = 0;
  my $tdy = 0;

  if (is_overlap($n,$n_hi)) {
    # double segment, spread apart
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);  # to the right
    $sdx += $rx;
    $sdy += $ry;
    $tdx += $rx;
    $tdy += $ry;
    $arrow = 'my harpoon';
  }
  $sx .= chamfer($sdx,'\MyOverlapGap');
  $sy .= chamfer($sdy,'\MyOverlapGap');
  $tx .= chamfer($tdx,'\MyOverlapGap');
  $ty .= chamfer($tdy,'\MyOverlapGap');

  $Printwrap = 4;
  printwrap(" \\draw[$arrow] ($sx,$sy)");
  $Printwrap += 4;
  printwrap("--($tx,$ty);");
}
printwrap_newline();
print "    % end generated\n";

exit 0;
