#!/usr/bin/perl -w

# Copyright 2015, 2016 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::BaseCnv 'cnv';
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'chamfer','xy_rotate_plus90';
use MyPrintwrap ':all';


my $level = 6;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::CCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

$level%2 == 0 or die;
my $rotates = (2 - $level/2) % 4;

$Printwrap = 2;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx,$y+$dy);
  my $n_other = $path->xyxy_to_n($x2,$y2, $x,$y);

  foreach (1 .. $rotates) {
    ($x,$y) = xy_rotate_plus90($x,$y);
    ($x2,$y2) = xy_rotate_plus90($x2,$y2);
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);
  }

  my $cx = chamfer($dx,'\MyShorten');
  my $cy = chamfer($dy,'\MyShorten');
  my $cx2 = chamfer(-$dx,'\MyShorten');
  my $cy2 = chamfer(-$dy,'\MyShorten');

  if (defined $n_other && $n_other < $n_hi) {
    ($dx,$dy) = xy_rotate_plus90($dx,$dy);

    $cx .= chamfer($dx,'\MyGap');
    $cy .= chamfer($dy,'\MyGap');
    $cx2 .= chamfer($dx,'\MyGap');
    $cy2 .= chamfer($dy,'\MyGap');
  }

  printwrap ("\\draw ($x$cx,$y$cy)--($x2$cx2,$y2$cy2);\n");
}

print "  % end generated\n";
exit 0;
